#if !defined(___Class_TAB2)
    #define  ___Class_TAB2

#include <iostream>
#include <utility>
#include <string>
//#include <deque>
//#include <vector>

#include "ReconResonance/RmatrixLimited/endf/CONT.h"
#include "ReconResonance/RmatrixLimited/endf/Numeric.h"
#include "ReconResonance/RmatrixLimited/endf/Document.h"

using namespace std;


class TAB2 {

 public:

  class Range {

   private:

    int nsept,inter;

   public:

    Range();
    Range( const int n, const int i );
   ~Range();

    int  NBT() const;
    int& NBT();
    int  INT() const;
    int& INT();

    bool operator==( const Range& o ) const;
    bool operator!=( const Range& o ) const;

  };

 private:

  CONT header;  deque<TAB2::Range> ary;

  public:

    TAB2();
    TAB2( const int nz, const int interpcode = 2 );
    TAB2( istream& is );
   
    virtual ~TAB2();

    TAB2           ( const TAB2& o );
    TAB2& operator=( const TAB2& o );

    Range  at        ( const int i ) const;
    Range& at        ( const int i );
    Range  operator[]( const int i ) const;
    Range& operator[]( const int i );

    Range  front() const;
    Range& front();
    Range  back () const;
    Range& back ();

    int  size () const;
    bool empty() const;

    void Store( istream& is );
    void Write( ostream& os = cout, const int MAT = 0, const int MF = 0, const int MT = 0, int* NS = NULL ) const;

    TAB2* Set( const CONT& o );
    TAB2* Set( const TAB2& o, const bool option = false );
    TAB2* Set( const int NZ, const int interpcode = 2 );
    //TAB2* Set( const deque<Range>& ranges );

    TAB2* C1( const double val );
    TAB2* C2( const double val );
    TAB2* L1( const int    val );
    TAB2* L2( const int    val );
    TAB2* NR( const int    val );
    TAB2* NZ( const int    val );

    double  C1() const;
    double& C1();
    double  C2() const;
    double& C2();
    int     L1() const;
    int&    L1();
    int     L2() const;
    int&    L2();
    int     NR() const;
    int&    NR();
    int     NZ() const;
    int&    NZ();

    int  NBT( const int i ) const;
    int& NBT( const int i );
    int  INT( const int i ) const;
    int& INT( const int i );

    bool Check() const;

    TAB2* Clear();
    TAB2* ClearArray();

    Range  Front() const { return this->front(); }
    Range  Back () const { return this->back (); }
    Range& Front()       { return this->front(); }
    Range& Back ()       { return this->back (); }

    int InterpCode( const int i ) const;

    TAB2* SetInterpCode( const int interpcode );

    TAB2* CutAbove( const int index );
    TAB2* CutBelow( const int index );

    //void Insert( const int index, const int inter = 2 );
    TAB2* PushFront( const int inter );
    TAB2* PushBack ( const int inter );

    TAB2* PushFront();
    TAB2* PushBack ();

    TAB2* PushFront( const TAB2& o );  // change NR, NZ, NBT and INT
    TAB2* PushBack ( const TAB2& o );  // change NR, NZ, NBT and INT

    TAB2* PopFront();
    TAB2* PopBack ();

    TAB2* Erase( const int n );

    void Diagnose( Document* result, int* ns ) const;

    bool operator==( const TAB2& o ) const;
    bool operator!=( const TAB2& o ) const;
    bool operator< ( const TAB2& o ) const;
    bool operator> ( const TAB2& o ) const;
    bool operator<=( const TAB2& o ) const;
    bool operator>=( const TAB2& o ) const;

    unsigned long NumberOfLines() const { return 1 + num_of_lines( this->NR(), 3 ); }

    ////ClassDef( TAB2, 1 );

};

istream& operator>>( istream& is,       TAB2& o );
ostream& operator<<( ostream& os, const TAB2& o );

#endif
