#if !defined(___Class_Level)
    #define  ___Class_Level

#include <vector>
#include "ReconResonance/RmatrixLimited/numeric/Parameter.h"
#include "ReconResonance/RmatrixLimited/rml/Coulomb.h"
#include "ReconResonance/RmatrixLimited/rml/Boundary.h"
#include "ReconResonance/RmatrixLimited/rml/Distant.h"

using namespace Numeric;

namespace RML {

class Exclusive : public Particle {

  bool opt;

 public:

  Exclusive();
 ~Exclusive();

  const bool is_true() const { return opt; }

  Exclusive* Set ( const Particle& o );
  bool       Find( const Particle& o ) const;

};



class Level : public Parameter, public Exclusive {  // Parameter -> eigenvalue

 public:

  class Branch : public Channel, public Parameter {  // Parameter -> reduced width amplitude

   public:

    Branch();
    Branch( const Channel& ch, const Parameter& value );
   ~Branch();

    void gamma( const Channel& ch, const Parameter& value );

    Parameter  gamma() const;
    Parameter& gamma();

    void ScaleEnergy( const double value );

    inline bool operator==( const Channel& ch ) const { return this->Channel::operator==( ch ); }
    inline bool operator!=( const Channel& ch ) const { return this->Channel::operator!=( ch ); }

    inline bool operator==( const TwoBody& o ) const { return this->TwoBody::operator==( o ); }
    inline bool operator!=( const TwoBody& o ) const { return this->TwoBody::operator!=( o ); }

    void Print( ostream& os = cout ) const;

  };

 private:

  vector<Branch> ary;

  Parameter Gam_g; /* to be used only for Reich-Moore approximation */ 

 public:

  Level();
  Level( const Parameter& eigenvalue );
 ~Level();

  const Branch& at( const int i ) const;
        Branch& at( const int i );

  Branch  operator[]( const int i ) const;
  Branch& operator[]( const int i );

  int size() const;

  Level* E( const Parameter& eigenvalue );

  Parameter  E() const;
  Parameter& E();

  Parameter* gamma( const Channel& ch, const double value );

  Level* gamma( const Channel& ch, const Parameter& pm );
  Level* gamma( const Channel& ch, const double value, const double error, const string name = "" );
  Level* gamma( const Channel& ch, const double value, const double error, const char error_unit, const string name = "" );

  Parameter  gamma( const Channel& ch ) const;

/*
  inline Parameter gamma( const Channel& ch ) const {

    for( int i=0; i<ary.size(); i++ ) if( ary[i] == ch ) return ary[i];

    return Parameter(0.0,0.0);

  }
*/
  Level*    Gamma( const Parameter& value );
  Level*    Gamma( const double value, const double error = 0.0, const string name = "" );

  Parameter  Gamma() const;
  Parameter& Gamma();

  Level* ScaleEnergy( const double fac );

  void CollectFreeParams( vector<Parameter*>* o );

  void Print( ostream& os = cout ) const;

};

ostream& operator<<( ostream& os, const Level::Branch& o );
ostream& operator<<( ostream& os, const Level& o );



class Structure : public Distant {

  vector<Level> ary;

 public:

  Structure();
  virtual ~Structure();

  Level  at( const int i ) const;
  Level& at( const int i );

  Level  operator[]( const int i ) const;
  Level& operator[]( const int i );

  int  size () const;
  bool empty() const;

  const Level::Branch& at( const int i, const int ii ) const;
        Level::Branch& at( const int i, const int ii );

  Level* ptr( const int i );

  Parameter  E( const int i ) const;
  Parameter& E( const int i );

  Level* E( const Parameter& eigenvalue );
  Level* E( const Parameter& eigenvalue, const Particle& exclusive );

  Level* E( const double eigenvalue, const double error = 0.0,                      const string name = "" );
  Level* E( const double eigenvalue,                     const Particle& exclusive, const string name = "" );
  Level* E( const double eigenvalue, const double error, const Particle& exclusive, const string name = "" );

  //Parameter  gamma( const int i, const Channel& ch ) const { return ary[i].gamma(ch); }

  Parameter gamma( const int i, const Channel& ch ) const;

  Parameter  Gamma( const int i ) const;
  Parameter& Gamma( const int i );

  void Set( const Structure& o );
  void Clear();

  void CollectFreeParams( vector<Parameter*>* o );

  Structure* ScaleEnergy( const double fac );

  Structure* ErrorPercent_E    (                    const double val );
  Structure* ErrorPercent_gamma(                    const double val );
  Structure* ErrorPercent_gamma( const Channel& ch, const double val );
  Structure* ErrorPercent_gamma( const Particle& a, const Nucleus& b, const double val );

  Structure* SetZero_gamma( const Particle& a, const Nucleus& b );

  Structure* Fix_E();
  Structure* Fix_gamma();
  Structure* Fix_gamma( const Channel& ch );
  Structure* Fix_gamma( const Particle& a, const Nucleus& b );
  Structure* Fix();

  void Print( ostream& os = cout ) const;


};

ostream& operator<<( ostream& os, const Structure& o );

}

#endif
