#!/bin/csh

set PWD = `pwd`
set SRC = ${PWD}/../../frendy/main
set EXE = ${SRC}/frendy.exe

set INP_DIR = ${PWD}/../input
set LIB_DIR = ${PWD}/../lib

set RUN_DIR = ${PWD}/process_result
set ACE_DIR = ${RUN_DIR}/ace

cd ${SRC}
make
cd ${PWD}

mkdir -p ${RUN_DIR}
mkdir -p ${ACE_DIR}

set LOG = ${RUN_DIR}/process_result.log
rm -rf ${LOG}

cd ${RUN_DIR}
ln -s ${LIB_DIR}  ./

foreach CASE \
  (  c01_ace_simple_hinh2o                      c01_ace_simple_u235                      \
     c01_ace_simple_u238                        c01_ace_simple_h001                      \
     c01_ace_simple_h002                        c01_ace_simple_c000                      \
     c01_ace_simple_o016                                                                 \
     c02_ace_graphite                           c02_ace_hinh2o                           \
     c02_ace_dind2o                             c02_ace_u235                             \
     c03_ace_like_njoy_beo                      c03_ace_like_njoy_u238                   \
     c11_mg_fast_reactor_from_ace_u235          c11_mg_fast_reactor_u235                 \
     c11_mg_from_ace_h001                       c11_mg_from_ace_u235                     \
     c11_mg_h001                                c11_mg_mod_group_structure_from_ace_u235 \
     c11_mg_mod_group_structure_u235            c11_mg_u235                              \
     c12_mg_from_ace_graphite                   c12_mg_from_ace_hinh2o                   \
     c12_mg_graphite                            c12_mg_hinh2o                            \
     c13_mg_from_ace_h2o                        c13_mg_from_ace_uo2                      \
     c13_mg_h2o                                 c13_mg_uo2                               \
     c14_mg_auto_bg_from_ace_u235               c14_mg_auto_bg_u235                      \
     c15_mg_auto_group_structure_from_ace_u235  c15_mg_auto_group_structure_u235         \
     c16_mg_ruc_0k_u235                         c16_mg_ruc_0k_u238                       \
     c16_mg_ruc_uo2 )

  echo ${CASE}

  set RUN_DIR_EACH = ${RUN_DIR}/${CASE}
  mkdir -p ${RUN_DIR_EACH}
  cd ${RUN_DIR_EACH}

  set LOG_EACH = ${RUN_DIR_EACH}/process_result.log

  if( ${CASE} == "c03_ace_like_njoy_beo" ) then
    cp -p ${LIB_DIR}/Be009.dat  ./tape20
    cp -p ${LIB_DIR}/27_beo.txt ./tape60
  else if( ${CASE} == "c03_ace_like_njoy_u238" ) then
    cp -p ${LIB_DIR}/U238.dat  ./tape20
  else if( ${CASE} == "c16_mg_ruc_uo2" ) then
    mv ../c16_mg_ruc_0k_u235/U235_RUC_RUC_92235.00c_MT2.mg ./U235_RUC_92235.00c_MT2.mg
    mv ../c16_mg_ruc_0k_u238/U238_RUC_RUC_92238.00c_MT2.mg ./U238_RUC_92238.00c_MT2.mg
  endif

  echo ${CASE} >> ${LOG}
  echo ""      >> ${LOG}
  ${EXE}  ${INP_DIR}/${CASE}.dat  > ${LOG_EACH}
  cat ${LOG}  ${LOG_EACH} > ${RUN_DIR}/tmp
  mv ${RUN_DIR}/tmp  ${LOG}
  echo ""      >> ${LOG}
  echo "======================================================================"  >> ${LOG}
  echo ""      >> ${LOG}

  if( ${CASE} == "c01_ace_simple_hinh2o" ) then
    mv 01_h_in_h2o.txt.ace    ${ACE_DIR}/01_h_in_h2o.ace
  else if( ${CASE} == "c01_ace_simple_u235" ) then
    mv U235.dat.ace           ${ACE_DIR}/U235.ace
  else if( ${CASE} == "c01_ace_simple_u238" ) then
    mv U238.dat.ace           ${ACE_DIR}/U238.ace
  else if( ${CASE} == "c01_ace_simple_h001" ) then
    mv H001.dat.ace           ${ACE_DIR}/H001.ace
  else if( ${CASE} == "c01_ace_simple_h002" ) then
    mv H002.dat.ace           ${ACE_DIR}/H002.ace
  else if( ${CASE} == "c01_ace_simple_c000" ) then
    mv C000.dat.ace           ${ACE_DIR}/C000.ace
  else if( ${CASE} == "c01_ace_simple_o016" ) then
    mv O016.dat.ace           ${ACE_DIR}/O016.ace
  else if( ${CASE} == "c02_ace_graphite" ) then
    mv 31_graphite.txt.ace    ${ACE_DIR}/31_graphite.ace
  else if( ${CASE} == "c02_ace_dind2o" ) then
    mv 11_d_in_d2o.txt.ace    ${ACE_DIR}/11_d_in_d2o.ace
  endif

  cd ${RUN_DIR}
end

